/*
 * bios_gio.c
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX1087

#include "bios_gio.h"
#include "../uwire.h"


#define AD_TIMEOUT          1000

static UINT8	pwc_iosel1 = 0;
static UINT8	pwc_ioout1 = 0;
static UINT8	pwc_iosel2 = 0;
static UINT8	pwc_ioout2 = 0;

/* --------------------------------------------------------------
    Function    : gioPwc_init
    Abstract    : Initialize UWIRE
	Input       : none
    Output      : none
   -------------------------------------------------------------- */
void gioPwc_init( void )
{
	uwire_init();
}

/* --------------------------------------------------------------
    Function    : gioPwc_exit
    Abstract    : Exit UWIRE
	Input       : none
    Output      : none
   -------------------------------------------------------------- */
void gioPwc_exit( void )
{
	uwire_exit();
}

/* --------------------------------------------------------------
    Function    : gioPwc_iosel
    Abstract    : Control IOSEL1/2
	Input       : reg	: IOSEL1 or IOSEL2
				  mask	: Bit in IOSEL1/2
				  onoff	: IN/OUT
    Output      : none
   -------------------------------------------------------------- */
void gioPwc_iosel( UINT8 reg, UINT8 mask, UINT8 onoff )
{
	UINT8 *stat;

	if(reg == PWC_GPIO_IOSEL1)
		stat = &pwc_iosel1;
	else if(reg == PWC_GPIO_IOSEL2)
		stat = &pwc_iosel2;
	else
		return;

	if(onoff)
		*stat |= mask;
	else
		*stat &= ~mask;

	uwire_write(UWIRE_CS0, reg, *stat);
}

/* --------------------------------------------------------------
    Function    : gioPwc_ioout
    Abstract    : Control IOOUT1/2
	Input       : reg	: IOOUT1 or IOOUT2
				  mask	: Bit in IOOUT1/2
				  onoff	: H/L
    Output      : none
   -------------------------------------------------------------- */
void gioPwc_ioout( UINT8 reg, UINT8 mask, UINT8 onoff )
{
	UINT8 *stat;

	if(reg == PWC_GPIO_IOOUT1)
		stat = &pwc_ioout1;
	else if(reg == PWC_GPIO_IOOUT2)
		stat = &pwc_ioout2;
	else
		return;

	if(onoff)
		*stat |= mask;
	else
		*stat &= ~mask;

	uwire_write(UWIRE_CS0, reg, *stat);
}

/* --------------------------------------------------------------
    Function    : gioPwc_write
    Abstract    : Control INTC
	Input       : reg	: Address of register
				  data	: Data
    Output      : none
	Note		: Use only ADINT when system boot
   -------------------------------------------------------------- */
void gioPwc_write( UINT8 reg, UINT8 data )
{
	uwire_write(UWIRE_CS0, reg, data);
}

/* --------------------------------------------------------------
    Function    : gioPwc_read
    Abstract    : Read data of register in PowerIC
	Input       : reg	: Address of register
	Output      : read data
   -------------------------------------------------------------- */
UINT8 gioPwc_read( UINT8 reg )
{
	int data;

	data = uwire_read(UWIRE_CS0, reg);

	return (UINT8)data;
}

/* --------------------------------------------------------------
    Function    : gioPwc_init_ad
    Abstract    : Initialize PowerIC A/D convertor
	Input       : none
	Output      : none
   -------------------------------------------------------------- */
void gioPwc_init_ad(void)
{
    /* initialize PWC AD mode */
    gioPwc_write(PWC_INTC_DETMOD3L, PWC_DETMOD3L_ADINT);
    /* reset mask for factor bit */
    gioPwc_write(PWC_INTC_FMASK3, PWC_INTC_FACT_ADINT);
}

/* --------------------------------------------------------------
    Function    : gioPwc_exit_ad
    Abstract    : Stop PowerIC A/D convertor
	Input       : none
	Output      : none
   -------------------------------------------------------------- */
void gioPwc_exit_ad(void)
{
    /* stop to request A/D */
    gioPwc_write(PWC_INTC_FMASK3, 0x00);
    /* clear interrupt factor */
    gioPwc_write(PWC_INTC_CLRFACT3, PWC_INTC_FACT_ADINT);
}

/* --------------------------------------------------------------
    Function    : gioPwc_read_ad
    Abstract    : Read PowerIC A/D convertor
	Input       : none
	Output      : Success/False:1/0
    Note		: BMASK3 has already been disabled in resume.c
   -------------------------------------------------------------- */
int gioPwc_read_ad(UINT8 ch, UINT8 *ad)
{
    int cnt;
    int rc = 0;

    /* clear interrupt factor */
    gioPwc_write(PWC_INTC_CLRFACT3, PWC_INTC_FACT_ADINT);

    /* request A/D */
    gioPwc_write(PWC_INTC_ADMODE, PWC_ADMODE_SINGLE|ch);
    /* wait for A/D finishes */
    cnt = AD_TIMEOUT;
    while( cnt ) {
        if( gioPwc_read(PWC_INTC_FACT3) & PWC_INTC_FACT_ADINT )
            break;
        cnt--;
    }

    /* read A/D data */
    if( cnt > 0 ) {
        *ad = gioPwc_read(PWC_INTC_ADDATA+(ch*2));
        rc = 1;
    }

    /* stop A/D */
    gioPwc_write(PWC_INTC_ADMODE, PWC_ADMODE_STOP|ch);

    return rc;
}

#endif    /* CONFIG_ICX1087 */

