/*
 * bios_lcd_epson.c
 *                 
 *
 * Copyright 2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/******************/
/*@ Include Files */
/******************/

#include <common.h>

#ifdef CONFIG_ICX1111

#include "icx1087.h"
#include "bios_lcd.h"
#include "bios_spi.h"
#include "bios_pwm.h"


/*******************/
/*@ LCD parameters */
/*******************/

/* Backlight parameters */
LcdParam_t lcd_param_epson = {
	.clk_sel    = SMU_PLLSEL_OSC,
	.clk_div    = 0x00000101,
	.bl_full    = 905,		/* higher than Lv.5 */
	.bl_inv     = 0,
};

static int  lcdInit_epson(void);
static void lcdExit_epson(void);
static void lcdStop_epson(void);

/* H/W operations */
LcdOps_t lcd_ops_epson = {
	.poweron	= lcdInit_epson,
	.poweroff	= lcdExit_epson,
	.stop		= lcdStop_epson,
};


/***********/
/*@ Macros */
/***********/

/* LCDC parameters */
#define	L5F30_HTOTAL_VAL	290				/* hblank = 50	*/
#define	L5F30_HAREA_VAL		LCD_HAREA_VAL
#define	L5F30_HEDGE1_VAL	30				/* hsync  = 10  */
#define L5F30_HEDGE2_VAL	40
#define	L5F30_VTOTAL_VAL	326				/* vblank = 6	*/
#define L5F30_VAREA_VAL		LCD_VAREA_VAL
#define	L5F30_VEDGE1_VAL	2				/* vsync  = 2	*/
#define L5F30_VEDGE2_VAL	4

/* start-byte */
#define	L5F30_SB_CMD		0x00700000		/* Set index register				*/
#define	L5F30_SB_DATA		0x00720000		/* Writes instruction or RAM data	*/

/* index registers */
#define	L5F30_IR_OSCCTL		0x0000
#define	L5F30_IR_DRVOUT		0x0001
#define	L5F30_IR_WAVCTL		0x0002
#define	L5F30_IR_ENTMOD		0x0003
#define	L5F30_IR_DISCTL1	0x0007
#define	L5F30_IR_DISCTL2	0x0008
#define	L5F30_IR_FRMCTL		0x000B
#define	L5F30_IR_IFCTL		0x000C
#define	L5F30_IR_PWRCTL1	0x0010
#define	L5F30_IR_PWRCTL2	0x0011
#define	L5F30_IR_PWRCTL4	0x0013
#define	L5F30_IR_PWRCTL5	0x0014
#define	L5F30_IR_SUBCTL		0x0015
#define	L5F30_IR_ADDSET1	0x0020
#define	L5F30_IR_ADDSET2	0x0021
#define	L5F30_IR_RAMWR		0x0022
#define	L5F30_IR_GAMCTL1	0x0030
#define	L5F30_IR_GAMCTL2	0x0031
#define	L5F30_IR_GAMCTL3	0x0032
#define	L5F30_IR_GAMCTL4	0x0033
#define	L5F30_IR_GAMCTL5	0x0034
#define	L5F30_IR_GAMCTL6	0x0035
#define	L5F30_IR_GAMCTL7	0x0036
#define	L5F30_IR_GAMCTL8	0x0037
#define	L5F30_IR_GAMCTL9	0x0038
#define	L5F30_IR_GAMCTL10	0x0039
#define	L5F30_IR_GSCAN		0x0040
#define	L5F30_IR_VSCTL		0x0041
#define	L5F30_IR_SCRPOS1	0x0042
#define	L5F30_IR_SCRPOS2	0x0043
#define	L5F30_IR_SCRPOS3	0x0044
#define	L5F30_IR_SCRPOS4	0x0045
#define	L5F30_IR_HADD		0x0046
#define	L5F30_IR_VADD1		0x0047
#define	L5F30_IR_VADD2		0x0048
#define	L5F30_IR_VWAKE		0x0050
#define	L5F30_IR_WK			0x0051
#define	L5F30_IR_SUBSEL		0x0060
#define	L5F30_IR_SUBWR		0x0061
#define	L5F30_IR_TEST1		0x0071
#define	L5F30_IR_TEST2		0x0072
#define	L5F30_IR_MIE		0x0073
#define	L5F30_IR_GPIOVAL	0x0075
#define	L5F30_IR_GPIOCTL	0x0076
#define	L5F30_IR_GPIOCLR	0x0077
#define	L5F30_IR_GPIOIE		0x0078
#define	L5F30_IR_GPIOPOL	0x0079
#define	L5F30_IR_LSCTL		0x0081
#define	L5F30_IR_MTPCTL		0x0090
#define	L5F30_IR_R92		0x0092

#define	L5F30_IR_WAIT		0xFFFF	/* unoffcial operation	*/


/***************/
/*@ Structures */
/***************/

typedef struct l5f30_command {
	unsigned short	ir;
	unsigned short	data;
} L5F30_COMMAND;


/********************/
/*@ Local Variables */
/********************/

/* Power ON sequence */
static L5F30_COMMAND l5f30_poweron_seq[] = {
	{L5F30_IR_OSCCTL,	0x0000},
	{L5F30_IR_IFCTL,	0x0110},
	{L5F30_IR_PWRCTL2,	0x3003},
	{L5F30_IR_PWRCTL5,	0x0022},
	{L5F30_IR_PWRCTL1,  0x1E20},
	{L5F30_IR_PWRCTL4,  0x0040},
	{L5F30_IR_WAIT,  	    10},	/* wait 10ms */
	{L5F30_IR_PWRCTL4,  0x0060},
	{L5F30_IR_WAIT,  	    50},	/* wait 50ms */
	{L5F30_IR_PWRCTL4,  0x0070},
	{L5F30_IR_WAIT,  	    40},	/* wait 40ms */
	{L5F30_IR_DRVOUT,   0x0927},
	{L5F30_IR_WAVCTL,	0x0700},
	{L5F30_IR_ENTMOD,	0x1030},
	{L5F30_IR_DISCTL1,	0x0800},
	{L5F30_IR_FRMCTL,   0x0020},
	{L5F30_IR_GAMCTL1,	0x0201},
	{L5F30_IR_GAMCTL2,	0x0607},
	{L5F30_IR_GAMCTL3,  0x0105},
	{L5F30_IR_GAMCTL4,  0x0400},
	{L5F30_IR_GAMCTL5,  0x0606},
	{L5F30_IR_GAMCTL6,  0x0305},
	{L5F30_IR_GAMCTL7,  0x0707},
	{L5F30_IR_GAMCTL8,  0x0003},
	{L5F30_IR_GAMCTL9,  0x1909},
	{L5F30_IR_GAMCTL10, 0x090F},
	{L5F30_IR_DISCTL1,  0x0812},
	{L5F30_IR_RAMWR,	0x0000},
	{L5F30_IR_WAIT,  	    35},	/* wait 35ms(2frames) */
	{L5F30_IR_DISCTL1,  0x0813},
	{L5F30_IR_RAMWR,	0x0000}
};
#define	NUM_POWERON_SEQ		(sizeof(l5f30_poweron_seq)/sizeof(l5f30_poweron_seq[0]))

/* Power OFF sequence */
static L5F30_COMMAND l5f30_poweroff_seq[] = {
	{L5F30_IR_DISCTL1,  0x0812},
	{L5F30_IR_WAIT,  	    35},	/* wait 35ms(2frames) */
	{L5F30_IR_DISCTL1,  0x0800},
	{L5F30_IR_PWRCTL1,  0x0600},
	{L5F30_IR_PWRCTL4,  0x0060},
	{L5F30_IR_WAIT,  	    50},	/* wait 50ms */
	{L5F30_IR_PWRCTL4,  0x0040},
	{L5F30_IR_WAIT,  	    50},	/* wait 50ms */
	{L5F30_IR_PWRCTL4,  0x0000},
	{L5F30_IR_WAIT,  	    30},	/* wait 30ms */
	{L5F30_IR_PWRCTL1,  0x0001}
};
#define	NUM_POWEROFF_SEQ	(sizeof(l5f30_poweroff_seq)/sizeof(l5f30_poweroff_seq[0]))

/** SPI configuration **/
static spi_config_t epson_spi_config = {
	SPI_SPI0,                                       /* SPI0             */
	SPI_CS0,                                        /* CS0              */
	SPI_BIT24,                                      /* 24bit            */
	SPI_SCLOCK_3MHz,                                /* 3MHz             */
	{SPI_MODE_MS_M, SPI_MODE_MS_M, SPI_MODE_MS_M},  /* all Master       */
	SPI0_POL_SET,                                   /* see. bios_spi.h  */
	{SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD}, /* 1word    */
	1                                               /* counter          */
};


/*******************/
/*@ Local Routines */
/*******************/

/*
 * Send a command & a data to LCD via SPI
 */
static int l5f30_send_command(L5F30_COMMAND *func)
{
	UINT32 buf;
	int rc = 0;

	if (func->ir == L5F30_IR_WAIT) {
		/* wait command */
		udelay((unsigned long)func->data*1000);
	}
	else {
		/* send a command */
		buf = L5F30_SB_CMD | func->ir;
		rc = spiWrite(&epson_spi_config, &buf);

		/* send a data */
		if ((rc == BIOS_OK) && (func->ir != L5F30_IR_RAMWR)) {
			buf = L5F30_SB_DATA | func->data;
			rc = spiWrite(&epson_spi_config, &buf);
		}
	}

	return rc;
}


/********************/
/*@ Global Routines */
/********************/

/*
 * Power ON 
 */
static int lcdInit_epson(void)
{
	unsigned long pullctl;
	int i;
	int rc = 0;

    /* CS_SEL pull control */
	pullctl = BIOS_MCR_INPW(SMU_PULLCTL_NAND);
	BIOS_MCR_OUTPW(SMU_PULLCTL_NAND, pullctl & ~SMU_PULLCTL_CS_SEL);

    /* BLTCTL pull control */
	pullctl = BIOS_MCR_INPW(SMU_PULLCTL_DSPDEBUG);
	BIOS_MCR_OUTPW(SMU_PULLCTL_DSPDEBUG, pullctl & ~SMU_PULLCTL_BLTCTL);

	/* start to supply system power */
	if (ICX11xx_CHK_SI_REV(ICX11xx_SI_REV_DOBER))
		gioPwc_write(PWC_PSCNT2, PWC_PSCNT2_REGGP4);
	udelay(2000);		/* wait more than 2ms */

	/* make a device reset */
	gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 1);

	udelay(20000);		/* wait more than 20ms */

    /* stop GatedClock */
	lcd_clkctrl_off();
	/* start PCLK/SCLK */
	lcd_clock_start(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_start(SMU_CLOCKGATE_LCD_CLK |
	                SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
	/* cancel reset */
	lcd_unreset_device();
	/* start GatedClock */
	lcd_clkctrl_on();

    /* clear frame buffer */
	memset((UINT8 *)LCD_FRAMEBUF_A, 0, LCD_FRAME_SIZE);
	memset((UINT8 *)LCD_FRAMEBUF_B, 0, LCD_FRAME_SIZE);

	/* initialize LCDC */
    /* set CONTROL register */
	BIOS_MCR_OUTPW(LCD_CONTROL, LCD_CONTROL_VAL|LCD_CONTROL_CLKPOL_FALL);
	/* set Display region register */
	BIOS_MCR_OUTPW(LCD_FRAMESEL,	LCD_FRAMESEL_VAL);
	BIOS_MCR_OUTPW(LCD_AREAADR_A,	LCD_FRAMEBUF_A);
	BIOS_MCR_OUTPW(LCD_AREAADR_B,	LCD_FRAMEBUF_B);
	BIOS_MCR_OUTPW(LCD_HOFFSET,		LCD_HOFFSET_VAL);
	BIOS_MCR_OUTPW(LCD_HTOTAL,		L5F30_HTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_HAREA,		L5F30_HAREA_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE1,		L5F30_HEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE2,		L5F30_HEDGE2_VAL);
	BIOS_MCR_OUTPW(LCD_VTOTAL,		L5F30_VTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_VAREA,		L5F30_VAREA_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE1,		L5F30_VEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE2,		L5F30_VEDGE2_VAL);
	/* clear interrupt status */
	BIOS_MCR_OUTPW(LCD_INTENCLR, LCD_INTSTATUS_VAL);
	BIOS_MCR_OUTPW(LCD_INTFFCLR, LCD_INTSTATUS_VAL);

    /* start SYNC */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);

    /* display setting */
	spiInit(&epson_spi_config);

	/* transfer commands */
	for (i = 0; i < NUM_POWERON_SEQ; i++) {
		rc = l5f30_send_command(&l5f30_poweron_seq[i]);
		if (rc != 0) {
			break;
		}
	}

	return rc;
}

/*
 * Power OFF 
 */
static void lcdExit_epson(void)
{
	int i;

	/* transfer commands */
	for (i = 0; i < NUM_POWEROFF_SEQ; i++)
		l5f30_send_command(&l5f30_poweroff_seq[i]);

	/* stop LCD */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_OFF);

	/* stop clock and reset */
	lcd_clock_stop(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_stop(SMU_CLOCKGATE_LCD_CLK |
	               SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
	lcd_reset_device();
	lcd_clkctrl_off();

	/* device reset */
    gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 0);		/* LCD_PON = Low	*/
}

/*
 * Turn off LCD
 */
static void lcdStop_epson(void)
{
	/* to be sync mode */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);
}

#endif /* CONFIG_ICX1111 */
