/*
 * bios_lcd_epson.c 
 *                     
 *
 * Copyright 2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/******************/
/*@ Include Files */
/******************/

#include <common.h>

#ifdef CONFIG_ICX1111

#include "icx1087.h"
#include "bios_lcd.h"
#include "bios_spi.h"
#include "bios_pwm.h"


/*******************/
/*@ LCD parameters */
/*******************/

/* Backlight parameters */
LcdParam_t lcd_param_tmd = {
	.clk_sel    = SMU_PLLSEL_OSC,
	.clk_div    = 0x00000101,
	.bl_full    = 1200,
	.bl_inv     = 1,
};

/* H/W operations */
static int  lcdInit_tmd(void);
static void lcdExit_tmd(void);
static void lcdStop_tmd(void);

LcdOps_t lcd_ops_tmd = {
	.poweron	= lcdInit_tmd,
	.poweroff	= lcdExit_tmd,
	.stop		= lcdStop_tmd,
};


/***********/
/*@ Macros */
/***********/

/* LCDC parameters */
#define	TMD_HTOTAL_VAL		280				/* hblank = 40	*/
#define	TMD_HAREA_VAL		LCD_HAREA_VAL
#define	TMD_HEDGE1_VAL		10				/* hsync  = 10  */
#define TMD_HEDGE2_VAL		20
#define	TMD_VTOTAL_VAL		326				/* vblank = 6	*/
#define TMD_VAREA_VAL		LCD_HEIGHT
#define	TMD_VEDGE1_VAL		2				/* vsync  = 2	*/
#define TMD_VEDGE2_VAL		4

/* start-byte */
#define	TMD_SB_CMD			0x00700000		/* Set index register				*/
#define	TMD_SB_DATA			0x00720000		/* Writes instruction or RAM data	*/

/* index registers */
#define	TMD_IR_DRVOUT		0x0001
#define	TMD_IR_WAVCTL		0x0002
#define	TMD_IR_PWRCTL1		0x0003
#define	TMD_IR_FRMCTL		0x000B
#define	TMD_IR_PWRCTL2		0x000C
#define	TMD_IR_PWRCTL3		0x000D
#define	TMD_IR_PWRCTL4		0x000E
#define	TMD_IR_GSCAN		0x000F
#define	TMD_IR_CLRMD		0x0010
#define	TMD_IR_HPRCH		0x0016
#define	TMD_IR_VPRCH		0x0017
#define	TMD_IR_PWRCTL5		0x001E
#define	TMD_IR_EXTCMD1		0x0028
#define	TMD_IR_EXTCMD2		0x0029
#define	TMD_IR_EXTCMD3		0x002D
#define	TMD_IR_GAMCTL1		0x0030
#define	TMD_IR_GAMCTL2		0x0031
#define	TMD_IR_GAMCTL3		0x0032
#define	TMD_IR_GAMCTL4		0x0033
#define	TMD_IR_GAMCTL5		0x0034
#define	TMD_IR_GAMCTL6		0x0035
#define	TMD_IR_GAMCTL7		0x0036
#define	TMD_IR_GAMCTL8		0x0037
#define	TMD_IR_GAMCTL9		0x003A
#define	TMD_IR_GAMCTL10		0x003B

#define	TMD_IR_WAIT			0xFFFF	/* unoffcial operation	*/


/***************/
/*@ Structures */
/***************/

typedef struct tmd_command {
	unsigned short	ir;
	unsigned short	data;
} TMD_COMMAND;


/********************/
/*@ Local Variables */
/********************/

/* Power ON sequence */
static TMD_COMMAND tmd_poweron_seq[] = {
	{TMD_IR_DRVOUT,		0x2B3F},
	{TMD_IR_WAVCTL,		0x0300},
	{TMD_IR_PWRCTL1,	0x6666},
	{TMD_IR_FRMCTL,		0xF480},
	{TMD_IR_PWRCTL2,	0x0002},
	{TMD_IR_PWRCTL3,	0x000C},
	{TMD_IR_PWRCTL4,	0x2D00},
	{TMD_IR_GSCAN,		0x0000},
	{TMD_IR_CLRMD,		0x00D4},
	{TMD_IR_HPRCH,		0xEF1C},
	{TMD_IR_VPRCH,		0x0003},
	{TMD_IR_PWRCTL5,	0x0000},
	{TMD_IR_GAMCTL1,	0x0000},
	{TMD_IR_GAMCTL2,	0x0000},
	{TMD_IR_GAMCTL3,	0x0007},
	{TMD_IR_GAMCTL4,	0x0500},
	{TMD_IR_GAMCTL5,	0x0007},
	{TMD_IR_GAMCTL6,	0x0707},
	{TMD_IR_GAMCTL7,	0x0707},
	{TMD_IR_GAMCTL8,	0x0005},
	{TMD_IR_GAMCTL9,	0x0002},
	{TMD_IR_GAMCTL10,	0x0002}
};
#define	NUM_POWERON_SEQ		(sizeof(tmd_poweron_seq)/sizeof(tmd_poweron_seq[0]))

/* Power OFF sequence */
static TMD_COMMAND tmd_poweroff_seq[] = {
	{TMD_IR_EXTCMD1,	0x0006},
	{TMD_IR_EXTCMD2,	0x0000},
	{TMD_IR_EXTCMD3,	0x3F66}
};
#define	NUM_POWEROFF_SEQ	(sizeof(tmd_poweroff_seq)/sizeof(tmd_poweroff_seq[0]))

/** SPI configuration **/
static spi_config_t tmd_spi_config = {
	SPI_SPI0,                                       /* SPI0             */
	SPI_CS0,                                        /* CS0              */
	SPI_BIT24,                                      /* 24bit            */
	SPI_SCLOCK_6MHz,                                /* 6MHz             */
	{SPI_MODE_MS_M, SPI_MODE_MS_M, SPI_MODE_MS_M},  /* all Master       */
	SPI0_POL_SET,                                   /* see. bios_spi.h  */
	{SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD}, /* 1word    */
	1                                               /* counter          */
};


/*******************/
/*@ Local Routines */
/*******************/

/*
 * Send a command & a data to LCD via SPI
 */
static int tmd_send_command(TMD_COMMAND *func)
{
	UINT32 buf;
	int rc = 0;

	if (func->ir == TMD_IR_WAIT) {
		/* wait command */
		udelay((unsigned long)func->data*1000);
	}
	else {
		/* send a command */
		buf = TMD_SB_CMD | func->ir;
		rc = spiWrite(&tmd_spi_config, &buf);

		/* send a data */
		if (rc == BIOS_OK) {
			buf = TMD_SB_DATA | func->data;
			rc = spiWrite(&tmd_spi_config, &buf);
		}
	}

	return rc;
}


/********************/
/*@ Global Routines */
/********************/

/*
 * Power ON 
 */
static int lcdInit_tmd(void)
{
	unsigned long pullctl;
	int i;
	int rc = 0;

    /* CS_SEL pull control */
	pullctl = BIOS_MCR_INPW(SMU_PULLCTL_NAND);
	BIOS_MCR_OUTPW(SMU_PULLCTL_NAND, pullctl & ~SMU_PULLCTL_CS_SEL);

	/* BLTCTL pull control */
	pullctl = BIOS_MCR_INPW(SMU_PULLCTL_DSPDEBUG);
	BIOS_MCR_OUTPW(SMU_PULLCTL_DSPDEBUG, pullctl & ~SMU_PULLCTL_BLTCTL);

	/* start to supply system power */
    if (ICX11xx_CHK_SI_REV(ICX11xx_SI_REV_DOBER))
		gioPwc_write(PWC_PSCNT2, PWC_PSCNT2_REGGP4);
	udelay(10000);		/* wait 10ms */

	/* make a device reset */
	gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 1);

	udelay(10000);		/* wait 10ms */

	/* display setting */
	spiInit(&tmd_spi_config);

	for (i = 0; i < NUM_POWERON_SEQ; i++) {
		rc = tmd_send_command(&tmd_poweron_seq[i]);
		if (rc != 0) {
			break;
		}
	}

    /* stop GatedClock */
	lcd_clkctrl_off();
	/* start PCLK/SCLK */
	lcd_clock_start(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_start(SMU_CLOCKGATE_LCD_CLK |
	                SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
	/* cancel reset */
	lcd_unreset_device();
	/* start GatedClock */
	lcd_clkctrl_on();

    /* clear frame buffer */
	memset((UINT8 *)LCD_FRAMEBUF_A, 0, LCD_FRAME_SIZE);
	memset((UINT8 *)LCD_FRAMEBUF_B, 0, LCD_FRAME_SIZE);

	/* initialize LCDC */
    /* set CONTROL register */
	BIOS_MCR_OUTPW(LCD_CONTROL, LCD_CONTROL_VAL|LCD_CONTROL_CLKPOL_FALL);
	/* set Display region register */
	BIOS_MCR_OUTPW(LCD_FRAMESEL,	LCD_FRAMESEL_VAL);
	BIOS_MCR_OUTPW(LCD_AREAADR_A,	LCD_FRAMEBUF_A);
	BIOS_MCR_OUTPW(LCD_AREAADR_B,	LCD_FRAMEBUF_B);
	BIOS_MCR_OUTPW(LCD_HOFFSET,		LCD_HOFFSET_VAL);
	BIOS_MCR_OUTPW(LCD_HTOTAL,		TMD_HTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_HAREA,		TMD_HAREA_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE1,		TMD_HEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE2,		TMD_HEDGE2_VAL);
	BIOS_MCR_OUTPW(LCD_VTOTAL,		TMD_VTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_VAREA,		TMD_VAREA_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE1,		TMD_VEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE2,		TMD_VEDGE2_VAL);
	/* clear interrupt status */
	BIOS_MCR_OUTPW(LCD_INTENCLR, LCD_INTSTATUS_VAL);
	BIOS_MCR_OUTPW(LCD_INTFFCLR, LCD_INTSTATUS_VAL);

    /* start SYNC */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);
	
	udelay(120000);		/* wait 120ms */

	return rc;
}

/*
 * Power OFF 
 */
static void lcdExit_tmd(void)
{
	int i;

    /* write black data */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);

	udelay(25000);		/* wait 25ms */

	/* transfer commands */
	for (i = 0; i < NUM_POWEROFF_SEQ; i++)
		tmd_send_command(&tmd_poweroff_seq[i]);

	udelay(50000);		/* wait 50ms */

    /* display data stop */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_OFF);

	/* stop clock and reset */
	lcd_clock_stop(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_stop(SMU_CLOCKGATE_LCD_CLK |
	               SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
	lcd_reset_device();
	lcd_clkctrl_off();

	/* device reset */
    gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 0);		/* LCD_PON = Low	*/
}

/*
 * Turn off LCD
 */
static void lcdStop_tmd(void)
{
	/* to be sync mode */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);
}

#endif /* CONFIG_ICX1111 */
