/*
 * bios_lcd_tmd_18qvga.c 
 *                       
 *
 * Copyright 2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/******************/
/*@ Include Files */
/******************/

#include <common.h>

#ifdef CONFIG_ICX1111

#include "icx1087.h"
#include "bios_lcd.h"
#include "bios_spi.h"
#include "bios_pwm.h"


/*******************/
/*@ LCD parameters */
/*******************/

/* Backlight parameters */
LcdParam_t lcd_param_tmd_18qvga = {
	.clk_sel    = SMU_PLLSEL_OSC,
	.clk_div    = 0x00000101,
	.bl_full    = 1080,
	.bl_inv     = 0,
};

/* H/W operations */
static int  lcdInit_tmd_18qvga(void);
static void lcdExit_tmd_18qvga(void);
static void lcdStop_tmd_18qvga(void);

LcdOps_t lcd_ops_tmd_18qvga = {
	.poweron	= lcdInit_tmd_18qvga,
	.poweroff	= lcdExit_tmd_18qvga,
	.stop		= lcdStop_tmd_18qvga,
};


/***********/
/*@ Macros */
/***********/

/* LCDC parameters */
#define	TMD_HTOTAL_VAL		290				/* hblank = 50	*/
#define	TMD_HAREA_VAL		LCD_HAREA_VAL
#define	TMD_HEDGE1_VAL		30				/* hsync  = 10  */
#define TMD_HEDGE2_VAL		40
#define	TMD_VTOTAL_VAL		326				/* vblank = 6	*/
#define TMD_VAREA_VAL		LCD_HEIGHT
#define	TMD_VEDGE1_VAL		2				/* vsync  = 2	*/
#define TMD_VEDGE2_VAL		4

/* start-byte */
#define	TMD_SB_CMD			0x00700000		/* Set index register				*/
#define	TMD_SB_DATA			0x00720000		/* Writes instruction or RAM data	*/

/* index registers */
#define	TMD_IR_HPRCH		0x0016
#define	TMD_IR_SWRST		0x0028
#define	TMD_IR_GAMRED1		0x0030
#define	TMD_IR_GAMRED2		0x0031
#define	TMD_IR_GAMRED3		0x0032
#define	TMD_IR_GAMRED4		0x0033
#define	TMD_IR_GAMGREEN1	0x0034
#define	TMD_IR_GAMGREEN2	0x0035
#define	TMD_IR_GAMGREEN3	0x0036
#define	TMD_IR_GAMGREEN4	0x0037
#define	TMD_IR_GAMBLUE1		0x0038
#define	TMD_IR_GAMBLUE2		0x0039
#define	TMD_IR_GAMBLUE3		0x003A
#define	TMD_IR_GAMBLUE4		0x003B
#define	TMD_IR_DRVOUT1		0x0052
#define	TMD_IR_PWRSEQCTL1	0x0057
#define	TMD_IR_PWRCTL2		0x005C
#define	TMD_IR_PWRCTL3		0x005D
#define	TMD_IR_PWRCTL4		0x005E
#define	TMD_IR_TIMINGSET1	0x005F
#define	TMD_IR_TIMINGSET3	0x0061
#define	TMD_IR_TIMINGSET5	0x0063
#define	TMD_IR_TIMINGSET6	0x0064
#define	TMD_IR_TIMINGSET7	0x0065
#define	TMD_IR_TIMINGSET8	0x0066


/***************/
/*@ Structures */
/***************/

typedef struct tmd_18qvga_command {
	unsigned short	ir;
	unsigned short	data;
} TMD_18QVGA_COMMAND;


/********************/
/*@ Local Variables */
/********************/

/* Power ON sequence */
static TMD_18QVGA_COMMAND tmd_18qvga_poweron_seq[] = {
	{TMD_IR_SWRST,		0x000E},
	{TMD_IR_HPRCH,		0x0014},
	{TMD_IR_PWRCTL2,	0x0077},
	{TMD_IR_PWRCTL3,	0xA7A4},
	{TMD_IR_PWRCTL4,	0x0004},
	{TMD_IR_TIMINGSET1,	0x0122},
	{TMD_IR_TIMINGSET3,	0x0111},
	{TMD_IR_TIMINGSET5,	0x0122},
	{TMD_IR_TIMINGSET6,	0x0009},
	{TMD_IR_TIMINGSET7,	0x0716},
	{TMD_IR_TIMINGSET8,	0x0304},
	{TMD_IR_DRVOUT1,	0x0002},
	{TMD_IR_GAMRED1,	0x0001},
	{TMD_IR_GAMRED2,	0xFF7D},
	{TMD_IR_GAMRED3,	0x8200},
	{TMD_IR_GAMRED4,	0x7DFF},
	{TMD_IR_GAMGREEN1,	0x0001},
	{TMD_IR_GAMGREEN2,	0xFF7D},
	{TMD_IR_GAMGREEN3,	0x8200},
	{TMD_IR_GAMGREEN4,	0x7DFF},
	{TMD_IR_GAMBLUE1,	0x0001},
	{TMD_IR_GAMBLUE2,	0xFF7D},
	{TMD_IR_GAMBLUE3,	0x8200},
	{TMD_IR_GAMBLUE4,	0x7DFF},
	{TMD_IR_PWRSEQCTL1,	0x0001}
};
#define	NUM_POWERON_SEQ		(sizeof(tmd_18qvga_poweron_seq)/sizeof(tmd_18qvga_poweron_seq[0]))

/* Power OFF sequence */
static TMD_18QVGA_COMMAND tmd_18qvga_poweroff_seq[] = {
	{TMD_IR_PWRSEQCTL1,	0x0000}
};
#define	NUM_POWEROFF_SEQ	(sizeof(tmd_18qvga_poweroff_seq)/sizeof(tmd_18qvga_poweroff_seq[0]))

/** SPI configuration **/
static spi_config_t tmd_18qvga_spi_config = {
	SPI_SPI0,                                       /* SPI0             */
	SPI_CS0,                                        /* CS0              */
	SPI_BIT24,                                      /* 24bit            */
	SPI_SCLOCK_3MHz,                                /* 3MHz             */
	{SPI_MODE_MS_M, SPI_MODE_MS_M, SPI_MODE_MS_M},  /* all Master       */
	SPI0_POL_SET,                                   /* see. bios_spi.h  */
	{SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD}, /* 1word    */
	1                                               /* counter          */
};


/*******************/
/*@ Local Routines */
/*******************/

/*
 * Send a command & a data to LCD via SPI
 */
static int tmd_18qvga_send_command(TMD_18QVGA_COMMAND *func)
{
	UINT32 buf;
	int rc = 0;

	/* send a command */
	buf = TMD_SB_CMD | func->ir;
	rc = spiWrite(&tmd_18qvga_spi_config, &buf);

	/* send a data */
	if (rc == BIOS_OK) {
		buf = TMD_SB_DATA | func->data;
		rc = spiWrite(&tmd_18qvga_spi_config, &buf);
	}

	return rc;
}


/********************/
/*@ Global Routines */
/********************/

/*
 * Power ON 
 */
static int lcdInit_tmd_18qvga(void)
{
	unsigned long pullctl;
	int i;
	int rc = 0;

    /* CS_SEL pull control */
	pullctl = BIOS_MCR_INPW(SMU_PULLCTL_NAND);
	BIOS_MCR_OUTPW(SMU_PULLCTL_NAND, pullctl & ~SMU_PULLCTL_CS_SEL);

	/* BLTCTL pull control */
	pullctl = BIOS_MCR_INPW(SMU_PULLCTL_DSPDEBUG);
	BIOS_MCR_OUTPW(SMU_PULLCTL_DSPDEBUG, pullctl & ~SMU_PULLCTL_BLTCTL);

	/* start to supply system power */
    if (ICX11xx_CHK_SI_REV(ICX11xx_SI_REV_DOBER))
		gioPwc_write(PWC_PSCNT2, PWC_PSCNT2_REGGP4);
	udelay(1000);		/* wait 1ms  */

	/* make a device reset */
	gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 1);

    /* stop GatedClock */
	lcd_clkctrl_off();
	/* start PCLK/SCLK */
	lcd_clock_start(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_start(SMU_CLOCKGATE_LCD_CLK |
	                SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
	/* cancel reset */
	lcd_unreset_device();
	/* start GatedClock */
	lcd_clkctrl_on();

    /* clear frame buffer */
	memset((UINT8 *)LCD_FRAMEBUF_A, 0, LCD_FRAME_SIZE);
	memset((UINT8 *)LCD_FRAMEBUF_B, 0, LCD_FRAME_SIZE);

	/* initialize LCDC */
    /* set CONTROL register */
	BIOS_MCR_OUTPW(LCD_CONTROL, LCD_CONTROL_VAL|LCD_CONTROL_CLKPOL_FALL);
	/* set Display region register */
	BIOS_MCR_OUTPW(LCD_FRAMESEL,	LCD_FRAMESEL_VAL);
	BIOS_MCR_OUTPW(LCD_AREAADR_A,	LCD_FRAMEBUF_A);
	BIOS_MCR_OUTPW(LCD_AREAADR_B,	LCD_FRAMEBUF_B);
	BIOS_MCR_OUTPW(LCD_HOFFSET,		LCD_HOFFSET_VAL);
	BIOS_MCR_OUTPW(LCD_HTOTAL,		TMD_HTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_HAREA,		TMD_HAREA_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE1,		TMD_HEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE2,		TMD_HEDGE2_VAL);
	BIOS_MCR_OUTPW(LCD_VTOTAL,		TMD_VTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_VAREA,		TMD_VAREA_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE1,		TMD_VEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE2,		TMD_VEDGE2_VAL);
	/* clear interrupt status */
	BIOS_MCR_OUTPW(LCD_INTENCLR, LCD_INTSTATUS_VAL);
	BIOS_MCR_OUTPW(LCD_INTFFCLR, LCD_INTSTATUS_VAL);

    /* start SYNC */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);
	udelay(10000);		/* wait 10ms  */

	/* display setting */
	spiInit(&tmd_18qvga_spi_config);

	for (i = 0; i < NUM_POWERON_SEQ; i++) {
		rc = tmd_18qvga_send_command(&tmd_18qvga_poweron_seq[i]);
		if (rc != 0) {
			break;
		}
	}

	return rc;
}

/*
 * Power OFF 
 */
static void lcdExit_tmd_18qvga(void)
{
	int i;

    /* write black data */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);

	udelay(40000);		/* wait 40ms(2 frame min.) */

	/* transfer commands */
	for (i = 0; i < NUM_POWEROFF_SEQ; i++)
		tmd_18qvga_send_command(&tmd_18qvga_poweroff_seq[i]);

	udelay(85000);		/* wait 85ms */

    /* display data stop */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_OFF);

	/* stop clock and reset */
	lcd_clock_stop(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_stop(SMU_CLOCKGATE_LCD_CLK |
	               SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
	lcd_reset_device();
	lcd_clkctrl_off();

	/* device reset */
    gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 0);		/* LCD_PON = Low	*/
}

/*
 * Turn off LCD
 */
static void lcdStop_tmd_18qvga(void)
{
	/* to be sync mode */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);
}

#endif /* CONFIG_ICX1111 */
