/*
 * bios_spi.c
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX1087

#include "bios_smu.h"
#include "bios_gio.h"
#include "bios_spi.h"


/* --------------------------------------------------------------
	Function	: spi_unreset_device
	Abstract	: Unreset SPI device
	Input		: none
	Return		: none
   -------------------------------------------------------------- */
static void spi_unreset_device( void )
{
	BIOS_MCR_OUTPW( SMU_RESETREQ1B_ENA, SMU_RESETDEVICE_SPI0 );
	BIOS_MCR_OUTPW( SMU_RESETREQ1B, SMU_RESETDEVICE_SPI0 );
	BIOS_MCR_OUTPW( SMU_RESETREQ1B_ENA, 0 );
	udelay( 100 );										/* 100u */
}

/* --------------------------------------------------------------
	Function	: spi_reset_device
	Abstract	: Reset SPI device
	Input		: none
	Return		: none
   -------------------------------------------------------------- */
static void spi_reset_device( void )
{
	BIOS_MCR_OUTPW( SMU_RESETREQ1B_ENA, SMU_RESETDEVICE_SPI0 );
	BIOS_MCR_OUTPW( SMU_RESETREQ1B, 0 );
	BIOS_MCR_OUTPW( SMU_RESETREQ1B_ENA, 0 );
}

/* --------------------------------------------------------------
	Function	: spi_clock_start
	Abstract	: Start SPI PCLK/SCLK
	Input		: mask	: start clock bit
	Return		: none
   -------------------------------------------------------------- */
static void spi_clock_start( UINT32 mask )
{
	BIOS_MCR_OUTPW( SMU_GCLKCTRL1ENA, mask );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL1, mask );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL1ENA, 0 );
}

/* --------------------------------------------------------------
	Function	: spi_clock_stop
	Abstract	: Stop SPI PCLK/SCLK
	Input		: mask	: stop clock bit
	Return		: none
   -------------------------------------------------------------- */
static void spi_clock_stop( UINT32 mask)
{
	BIOS_MCR_OUTPW( SMU_GCLKCTRL1ENA, mask );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL1, 0 );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL1ENA, 0 );
}

/* --------------------------------------------------------------
    Function    : spi_clkctrl_on
    Abstract    : Start SPI0 PCLK,SCLK of GatedClock
    Input       : none
    Return      : none
   -------------------------------------------------------------- */
static void spi_clkctrl_on( void )
{
    UINT32 regval, ahbclkctrl1;

    /* PCLK */
    ahbclkctrl1 = BIOS_MCR_INPW( SMU_AHBCLKCTRL1 );
    BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 & ~(SMU_CLKCTRL_PB0|SMU_CLKCTRL_PB1) );
    regval = BIOS_MCR_INPW( SMU_APBCLKCTRL );
    regval |= SMU_CLKCTRL_SPI0PCLK;
    BIOS_MCR_OUTPW( SMU_APBCLKCTRL, regval );
    BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 );
    /* SCLK */
    regval = BIOS_MCR_INPW( SMU_CLKCTRL );
    regval |= SMU_CLKCTRL_SPI0SCLK;
    BIOS_MCR_OUTPW( SMU_CLKCTRL, regval );
}

/* --------------------------------------------------------------
    Function    : spi_clkctrl_off
    Abstract    : Stop SPI0 PCLK,SCLK of GatedClock
    Input       : none
    Return      : none
   -------------------------------------------------------------- */
static void spi_clkctrl_off( void )
{
    UINT32 regval, ahbclkctrl1;

    /* PCLK */
    ahbclkctrl1 = BIOS_MCR_INPW( SMU_AHBCLKCTRL1 );
    BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 & ~(SMU_CLKCTRL_PB0|SMU_CLKCTRL_PB1) );
    regval = BIOS_MCR_INPW( SMU_APBCLKCTRL );
    regval &= ~SMU_CLKCTRL_SPI0PCLK;
    BIOS_MCR_OUTPW( SMU_APBCLKCTRL, regval );
    BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 );
    /* SCLK */
    regval = BIOS_MCR_INPW( SMU_CLKCTRL );
    regval &= ~SMU_CLKCTRL_SPI0SCLK;
    BIOS_MCR_OUTPW( SMU_CLKCTRL, regval );
}

/* --------------------------------------------------------------
	Function	: spiInit
	Abstract	: Initialize SPI
	Input		: pConfig		 : pointer to SPI config structure
	Return		: BIOS_OK		 : success
				  BIOS_ERR_PARAM : invalid parameter(not used)
	Note		: PLL_SEL has been selected to HPLL in poweron.S
   -------------------------------------------------------------- */
BIOS_RESULT spiInit( spi_config_t *pConfig )
{
    UINT32 div_sclk;

	/* stop PCLK/SCLK */
	spi_clock_stop(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);
	/* set the divisor of system clock */
	div_sclk = BIOS_CLOCK_HPLL / pConfig->clock - 1;
	BIOS_MCR_OUTPW(SMU_SPI0_SCLK_DIV, div_sclk);
	/* start PCLK/SCLK */
	spi_clock_start(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);
	/* auto clock control off */
	spi_clkctrl_off();

	/* cancel reset */
	spi_unreset_device();

	/* auto clock control on */
	spi_clkctrl_on();

	/* stop PCLK/SCLK */
	spi_clock_stop(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);

	return BIOS_OK;
}

/* --------------------------------------------------------------
	Function	: spiEnd
	Abstract	: Finish SPI
	Input		: pConfig		 : pointer to SPI config structure
	Return		: BIOS_OK        : success
				  BIOS_ERR_PARAM : invalid parameter(not used)
   -------------------------------------------------------------- */
BIOS_RESULT spiEnd( void )
{
	/* start PCLK/SCLK */
	spi_clock_start(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);

	/* reset */
	spi_reset_device();

	/* stop PCLK/SCLK */
	spi_clock_stop(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);

	return BIOS_OK;
}

/* --------------------------------------------------------------
	Function	: spiWrite
	Abstract	: Write to SPI
	Input		: *pConfig	: pointer to SPI configuration struct
				  *data		: pointer to data region
	Return		: BIOS_OK			: success
				  BIOS_ERR_CONFIG	: invalid configuration
			  	  BIOS_ERR_STATE	: invalid status
				  BIOS_ERR_TIMEOUT	: timeout
   -------------------------------------------------------------- */
BIOS_RESULT spiWrite( spi_config_t *pConfig, UINT32 *data )
{
	UINT32 reg;
	UINT32 spi_tick;
	UINT32	local_counter;
	const UINT32	TRANS_BIT[4] = {SPI_NB_BIT8,SPI_NB_BIT16,SPI_NB_BIT24,SPI_NB_BIT32};
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	UINT8 cs_num = pConfig->cs_num;
#endif

	/* error check */
	if(pConfig->spi_num == SPI_SPI0){
		/* parameter check */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
		if((pConfig->cs_num == SPI_CS0) ||
		   (pConfig->cs_num == SPI_CS1) ||
		   (pConfig->cs_num == SPI_CS2)) {
			BIOS_MCR_OUTPH(GPIO_GPIO2_EO, GPIO_GPIO2_CS_SEL);
			BIOS_MCR_OUTPW(GPIO_GPIO2_DO,
						   (GPIO_GPIO2_CS_SEL<<16)|((~GPIO_GPIO2_CS_SEL)&0xFFFF));
		}
		else if(pConfig->cs_num == SPI_CS3) {
			cs_num = SPI_CS0;
			BIOS_MCR_OUTPH(GPIO_GPIO2_EO, GPIO_GPIO2_CS_SEL);
			BIOS_MCR_OUTPW(GPIO_GPIO2_DO,
						   (GPIO_GPIO2_CS_SEL<<16)|(GPIO_GPIO2_CS_SEL&0xFFFF));
		}
		else {
			return BIOS_ERR_CONFIG;
		}
#else
		if((pConfig->cs_num != SPI_CS0) && (pConfig->cs_num != SPI_CS1) &&
		   (pConfig->cs_num != SPI_CS2) ){
			return BIOS_ERR_CONFIG;
		}
#endif
	}
	else{
		return BIOS_ERR_CONFIG;
	}

	/* parameter check */
	if((pConfig->bit_num != SPI_BIT8) && (pConfig->bit_num != SPI_BIT16) && 
	   (pConfig->bit_num != SPI_BIT24) && (pConfig->bit_num != SPI_BIT32)){
		return BIOS_ERR_CONFIG;
	}
	/* end parameter check*/

	/* start the clock */
	spi_clock_start(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);

	/* clear SPI register */
	BIOS_MCR_OUTPW(MP_SP0_FFCLR, SPI_FFCLR_ALLERR);
	BIOS_MCR_OUTPW(MP_SP0_ENCLR, SPI_ENCLR_ALLERR);

	/* set PCTL_SPI */
	reg = BIOS_MCR_INPW(SMU_PCTL_SPI);
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	if(pConfig->mode[cs_num] == SPI_MODE_MS_M){
#else
	if(pConfig->mode[pConfig->cs_num] == SPI_MODE_MS_M){
#endif
		reg &= ~(0x2 << pConfig->spi_num);
	}else{
		reg |= (0x2 << pConfig->spi_num);
	}
	BIOS_MCR_OUTPW(SMU_PCTL_SPI, reg);

	/* set SPI register mode */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	BIOS_MCR_OUTPW(MP_SP0_MODE, 
				   (pConfig->mode[cs_num] |
				    (TRANS_BIT[pConfig->bit_num] << 8) |
				    (cs_num << 4)) );
#else
	BIOS_MCR_OUTPW(MP_SP0_MODE, 
				   (pConfig->mode[pConfig->cs_num] |
				    (TRANS_BIT[pConfig->bit_num] << 8) |
				    (pConfig->cs_num << 4)) );
#endif

	/* set SPI register pol*/
	BIOS_MCR_OUTPW(MP_SP0_POL, pConfig->pol);

	/* set SPI register data transmit cycle */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	BIOS_MCR_OUTPW(MP_SP0_TX_TIM, pConfig->tx_tim[cs_num]);
#else
	BIOS_MCR_OUTPW(MP_SP0_TX_TIM, pConfig->tx_tim[pConfig->cs_num]);
#endif

	/* reset */
	reg = BIOS_MCR_INPW(MP_SP0_CONTROL);
	reg |= CONTROL_RST;
	BIOS_MCR_OUTPW(MP_SP0_CONTROL, reg);
	udelay(1);			/* 6MHz:1us wait/1500kHz:3us wait */
	reg &= ~CONTROL_RST;
	BIOS_MCR_OUTPW(MP_SP0_CONTROL, reg);

	for ( local_counter = 0; local_counter < pConfig->counter; local_counter++)
	{
		/* set transmit data*/
		BIOS_MCR_OUTPW(MP_SP0_TX_DATA, data[local_counter]);

		/* start to transmit */
		BIOS_MCR_OUTPW(MP_SP0_CONTROL, CONTROL_WRT);
		BIOS_MCR_OUTPW(MP_SP0_CONTROL, (CONTROL_WRT | CONTROL_START));

		/* wait finish to transmit(poling) */
		spi_tick = 0;
		while ((BIOS_MCR_INPW(MP_SP0_CONTROL) & CONTROL_START) != 0) {
			if (++spi_tick > SPI_MAX_WAIT) {
				/* timeout */
				reg = BIOS_MCR_INPW(MP_SP0_CONTROL);
				reg |= CONTROL_RST;
				BIOS_MCR_OUTPW(MP_SP0_CONTROL, reg);

				/* stop the clock */
				spi_clock_stop(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);

				return BIOS_ERR_TIMEOUT;
			}
		}
	}

	/* reset */
	reg = BIOS_MCR_INPW(MP_SP0_CONTROL);
	reg |= CONTROL_RST;
	BIOS_MCR_OUTPW(MP_SP0_CONTROL, reg);

	/* stop the clock */
	spi_clock_stop(SMU_CLOCKGATE_SPI0_PCLK | SMU_CLOCKGATE_SPI0_SCLK);

	return BIOS_OK;
}

#endif	/* CONFIG_ICX1087 */

