/*
 * icx1087_lcd.c
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX1087

#include "bios_lcd.h"
#include "bios_pwm.h"
#include "icx1087.h"


/**********/
/* Macros */
/**********/
#define	LCD_FADEIN_INTVAL_F	500		/* 500us	*/
#define	LCD_FADEIN_INTVAL	5000	/* 5ms		*/
#define	LCD_FADEIN_CNT		100		/* 100ms	*/

/********************/
/* Local variables */
/********************/
static int lcd_initialized = 0;

/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
LcdParam_t *icx11xx_lcd_param;

static LcdOps_t   *lcd_ops = NULL;
#endif

/******************/
/* Local routines */
/******************/

#if (CFG_LCD_FORMAT != CFG_DATA_FORMAT)
/******************************************************************************
* FUNCTION : convert RGB565 data to RGB666.
* INPUT    : rgb565... one pixel data of RGB565
* RETURN   : one pixel data of RGB666
* NOTE     : none
* CREATE   : 2006.09.14 M.Itawaki
* UPDATE   :
******************************************************************************/
static unsigned long convertRGB565toRGB666Sub(unsigned long rgb565)
{
    unsigned long rgb666=0;

    rgb666 =
        (((rgb565 & 0x07ff) << 1) | ((rgb565 & 0x0010) >> 4))   /*Blue & Green*/
        | (((rgb565 & 0xF800) << 2) | ((rgb565 & 0x8000) >> 3));   /*Red*/
    return rgb666;

}

/******************************************************************************
* FUNCTION : convert RGB565 data to RGB666.
* INPUT    : *src...address of RGB565
*            *dst...address of RGB666
*            *size...numbers of pixel(QVGA...320x240)
* NOTE     : none
* CREATE   : 2006.09.14 M.Itawaki
* UPDATE   :
******************************************************************************/
static void convertRGB565toRGB666(unsigned short *src, unsigned long *dst, int size)
{
    int i;
    int index;
    unsigned long rgb565;
    unsigned long rgb666;
    unsigned long shift_data;

    *dst = 0;                  /*init destination area*/
    index = 0;
    for(i=0 ; i < size ; i++){
        //printk("src=0x%x,dst=0x%x,(%d)\n",*src,*dst,i);
        rgb565 = (unsigned long) *src;
        rgb666 = convertRGB565toRGB666Sub(rgb565);
        shift_data = rgb666 << index;
        *dst |= shift_data;
        if(index > 14){   /*when rgb565 crosses a boundary of 32bit*/
            shift_data = rgb666 >> (32 - index);
            dst++;
            *dst = 0;
            *dst |= shift_data;
        }
        index = (index + 18) & 0x1F;           /*RGB666: 18bit/pixel*/
        if(index == 0){                    /*if index=0, change destination*/
            dst++;
            *dst = 0;
        }
        src++;
    }
}
#endif


/*******************/
/* Global routines */
/*******************/

/* 2007.03.06 fuj : Applied various LCD models */
#ifdef CONFIG_ICX1111
/**
 * Initialize LCD
 *   Called from board_init(mp200.c)
 *   Input  : none
 *   Output : none
 */
void icx11xx_init_lcd( void )
{
	if (ICX11xx_CHK_SI_LCD(ICX11xx_SI_LCD_EPSON)) {		/* EPSON */
		lcd_ops = &lcd_ops_epson;
		icx11xx_lcd_param = &lcd_param_epson;
	}
	else if (ICX11xx_CHK_SI_LCD(ICX11xx_SI_LCD_TMD)) {	/* TMD */
		lcd_ops = &lcd_ops_tmd;
		icx11xx_lcd_param = &lcd_param_tmd;
	}
	else if (ICX11xx_CHK_SI_LCD(ICX11xx_SI_LCD_TMD_18QVGA)) {
		lcd_ops = &lcd_ops_tmd_18qvga;					/* TMD	*/
		icx11xx_lcd_param = &lcd_param_tmd_18qvga;
	}
	else {												/* default			*/
		lcd_ops = &lcd_ops_sony; 						/* SONY	*/
		icx11xx_lcd_param = &lcd_param_sony;
	}

	/* set the clock source */
	BIOS_MCR_OUTPW(SMU_LCD_LCLK_SEL, icx11xx_lcd_param->clk_sel);
	BIOS_MCR_OUTPW(SMU_LCD_LCLK_DIV, icx11xx_lcd_param->clk_div);
}
#endif

/**
 * Display LCD
 *   Called from board_init(mp200.c)
 *   Input  : img_no	: Kind of image
 *          : disp_on   : Do or not initialize
 *   Output : none
 */
void icx1087_disp_lcd( UINT8 img_no, UINT8 disp_on )
{
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	int cnt;
	int interval;

	/* power on LCD */
	if (!lcd_initialized && disp_on) {
		/* initialize LCD & PWM */
		if (lcd_ops) {
		   if (lcd_ops->poweron() == BIOS_OK) {
				pwmInit();
				lcd_initialized = 1;
			}
			else {
				lcd_ops->poweroff();
			}
		}
	}

	/* get image data */
#if (CFG_LCD_FORMAT == CFG_DATA_FORMAT)
	if (nand_get_image(img_no, (void *)LCD_FRAMEBUF_A, LCD_DATA_SIZE)) {
#else
	if (nand_get_image(img_no, (void *)LCD_DATABUF, LCD_DATA_SIZE)) {
		/* convert RGB565 to RGB666 */
		convertRGB565toRGB666((unsigned short *)LCD_DATABUF,
							  (unsigned long  *)LCD_FRAMEBUF_A,
							  (LCD_WIDTH * LCD_HEIGHT));
#endif

		if (lcd_initialized && disp_on) {
			/* turn on backlight */
			lcdStart();
			if (img_no == LCD_IMG_OPENING)
				interval = LCD_FADEIN_INTVAL;
			else
				interval = LCD_FADEIN_INTVAL_F;

			/* fade in BL */
			for( cnt = 0; cnt <= LCD_FADEIN_CNT; cnt++ ) {
				pwmSetValue(cnt);
				udelay(interval);
				if( cnt == 0 ) lcdCtrlBL(1);
			}
		}
	}
	else {
		/* fail to read image data */
#if (CFG_LCD_FORMAT != CFG_DATA_FORMAT)
		memset((void *)LCD_DATABUF, 0, LCD_DATA_SIZE);
#endif
	}
#else	/* CONFIG_ICX1111 */
	UINT32 frame;
	UINT32 frameadr;
	int cnt;

	if (lcd_initialized == 0) {

		if (disp_on) {
			/* initialize LCD & PWM */
			lcdInit();
			pwmInit();

			lcd_initialized = 1;
		}

		/* get image data */
#if (CFG_LCD_FORMAT == CFG_DATA_FORMAT)
		if (nand_get_image(img_no, (void *)LCD_FRAMEBUF_A, LCD_DATA_SIZE)) {
#else
		if (nand_get_image(img_no, (void *)LCD_DATABUF, LCD_DATA_SIZE)) {
			/* convert RGB565 to RGB666 */
			convertRGB565toRGB666((unsigned short *)LCD_DATABUF,
								  (unsigned long  *)LCD_FRAMEBUF_A,
								  (LCD_WIDTH * LCD_HEIGHT));
#endif

			if (disp_on) {
				/* turn on backlight */
				if (img_no == LCD_IMG_OPENING) {
					/* fade in BL */
					for( cnt = 0; cnt <= LCD_FADEIN_CNT; cnt++ ) {
						pwmSetValue(cnt);
						udelay(LCD_FADEIN_INTVAL);
						if( cnt == 0 ) lcdCtrlBL(1);
					}
				}
				else {
					/* turn on BL */
					pwmSetValue(PWM_BL_MAX);	
					lcdCtrlBL(1);
				}
			}
		}
		else {
			/* fail to read image data */
/* 2006.11.30 fuj : modified timing of power on for LCD */
/*			lcdStop(); */
/*			pwmStop(); */

#if (CFG_LCD_FORMAT != CFG_DATA_FORMAT)
			memset((void *)LCD_DATABUF, 0, LCD_DATA_SIZE);
#endif

			lcd_initialized = 0;
		}
	}
	else {
		/* get address of frame */
		frame = lcdGetFrameNo();
		frameadr = (frame == LCD_FRAMESEL_A) ? LCD_FRAMEBUF_B : LCD_FRAMEBUF_A; 

		/* get source image from NAND */
#if (CFG_LCD_FORMAT == CFG_DATA_FORMAT)
		if (nand_get_image(img_no, (void *)frameadr, LCD_DATA_SIZE)) {
#else
		if (nand_get_image(img_no, (void *)LCD_DATABUF, LCD_DATA_SIZE)) {
			/* convert RGB565 to RGB666 */
			convertRGB565toRGB666((unsigned short *)LCD_DATABUF,
								  (unsigned long  *)frameadr,
								  (LCD_WIDTH * LCD_HEIGHT));
#endif
			lcdChangeFrame();
			pwmSetValue(PWM_BL_MAX);
			lcdCtrlBL(1);	 				/* just in case */
		}
	}
#endif	/* CONFIG_ICX1111 */
	return;
}

/**
 * turn off LCD
 *   Called from board_init(mp200.c)
 *   Input  : none
 *   Output : none
 */
void icx1087_off_lcd(void)
{
	lcdCtrlBL(0);
	pwmSetValue(PWM_BL_MIN);
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	if (lcd_ops)
	   lcd_ops->stop();
#else
	lcdStop();
#endif
	pwmStop();
/* 2007.03.06 fuj : Applied various models */
#ifndef CONFIG_ICX1111
	lcd_initialized = 0;
#endif
}

/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
/**
 * shutdown LCD
 *   Called from icx1087_power_off(icx1087_pm.c)
 *   Input  : none
 *   Output : none
 */
void icx11xx_exit_lcd(void)
{
	if (lcd_ops)
		lcd_ops->poweroff();

	pwmExit();
}
#endif

#endif /* CONFIG_ICX1087 */
